@@ -1,21 +1,20 @@
 #' Transformation of the raw data
 #'
 #' Function that transforms the raw data from REDCap read by the function `redcap_data`. It returns the transformed data and dictionary along with the summary of the results of each step.
-#' @param ... List containing the data and the dictionary and the event_form if it's needed. Can be the output of the function `redcap_data`.
-#' @param data Data frame containing data from REDCap. If the list is specified this argument is not needed.
-#' @param dic Data frame  containing the dictionary read from REDCap. If the list is specified this argument is not needed.
-#' @param event_form Data frame  containing the correspondence of each event with each form. If the list is specified this argument is not needed.
-#' @param event_path Character string with the pathname of the file with the correspondence between each event and each form (it can be downloaded in `Designate Instruments for My Events` inside the `Project Setup` section of REDCap).
+#' @param ... Output of the function `redcap_data`, that is a list containing the data frames of the data, he dictionary and the event_form (if it's needed) of the REDCap project.
+#' @param data Data frame containing the data read from REDCap. If the list is specified this argument is not necessary.
+#' @param dic Data frame  containing the dictionary read from REDCap. If the list is specified this argument is not necessary.
+#' @param event_form Data frame containing the correspondence of each event with each form. If the list is specified this argument is not necessary.
 #' @param checkbox_labels Character vector with the names that will have the two options of every checkbox variable. Default is `c('No', 'Yes')`.
-#' @param checkbox_na Logical indicating if values of checkboxes that have a branching logic have to set to missing only when the branching logic is missing (meaning that some of the variables specified in it are missing) or also when the branching logic isn't satisfied (true).
+#' @param checkbox_na Logical indicating if values of checkboxes that have a branching logic have to set to missing only when the branching logic is missing (if set to false) or also when the branching logic isn't satisfied (if set to true). The default is false.
+#' @param exclude_recalc Character vector with the names of the variables that do not have to be recalculated. Might be useful for projects were there are some calculated fields that have a time consuming recalculation.
 #' @param exclude_to_factor Character vector with the names of the variables that do not have to be transformed to factors.
-#' @param keep_labels Logical indicating if the labels have to be kept or not.
-#' @param delete_vars Character vector specifying the pattern that will contain variables to exclude. By default, variables ending up with `_complete` will be removed.
+#' @param delete_vars Character vector specifying the regex pattern that will contain the variables to exclude. By default, variables ending up with `_complete` and `_timestamp` will be removed.
 #' @param final_format Character string indicating the final arrangement format of the data that the function will return. Choose one of `raw`, `by_event` or `by_form`. `raw` (default) will return the transformed data with the original structure. `by_event` will return the transformed data as a nested data frame by event. `by_form` will return the transformed data as a nested data frame by form.
 #' @param which_event Character string indicating if only one event has to be returned if the final format selected is `by_event`.
 #' @param which_form Character string indicating if only one form has to be returned if the final format selected is `by_form`.
 #' @param wide Logical indicating if the data split by form (if selected) has to be in a wide format or in a long one.
-#' @return List with the transformed dataset, dictionary and the results
+#' @return List with the transformed dataset, dictionary, event_form and the results
 #'
 #' @examples
 #' rd_transform(covican)
@@ -26,7 +25,7 @@
 #'
 #' @export
 
-rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_path = NULL, checkbox_labels = c("No", "Yes"), checkbox_na = FALSE, exclude_to_factor = NULL, keep_labels = FALSE, delete_vars = c("_complete", "_timestamp"), final_format = "raw", which_event = NULL, which_form = NULL, wide = NULL){
+rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, checkbox_labels = c("No", "Yes"), checkbox_na = FALSE, exclude_recalc = NULL, exclude_to_factor = NULL, delete_vars = c("_complete", "_timestamp"), final_format = "raw", which_event = NULL, which_form = NULL, wide = NULL){
 
   results <- NULL
   ind <- 1
@@ -85,45 +84,59 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
     stop("final_format argument has to be one of the following: 'raw', 'by_event', 'by_form'", call. = FALSE)
   }
 
-  #If an event_path is provided, read the event file
-  if(!is.null(event_path)){
-    if(!is.null(event_form)){
-      warning("The event-form has been specified twice so the function will not read the file in the event_path argument.")
-    }else{
-      event_form <- utils::read.csv(paste(event_path), encoding="UTF-8")
-    }
-  }
-
   #If the project is longitudinal and the event hasn't been specified:
   if(longitudinal & is.null(event_form)){
     warning("The project contains more than one event. For a complete transformation is recommended to include the event-form correspondence")
   }
 
+  message("Transformation in progress...")
 
   labels <- Hmisc::label(data)
 
-  #Remove all variable labels:
+  #Change the labelled class of each column but don't remove the label:
   data <- data %>%
     dplyr::mutate_all(function(x){
       class(x) <- setdiff(class(x), "labelled")
-      attr(x,"label") <- NULL
+      # attr(x,"label") <- NULL
       x
     })
 
+  #Change the format of dates
+  #Identify dates that have the tag "date_"/"datetime_"/"datetime_seconds_" in redcap. It will have always the format "Y-M-D" in any case
+  var_date <- dic %>%
+    dplyr::filter(grepl("^date_", .data$text_validation_type_or_show_slider_number)) %>%
+    dplyr::pull(.data$field_name)
+
+  var_datetime <- dic %>%
+    dplyr::filter(grepl("^datetime_", .data$text_validation_type_or_show_slider_number)) %>%
+    dplyr::pull(.data$field_name)
+
+  data <- data %>%
+    dplyr::mutate_at(var_date, as.Date) %>%
+    # '' values don't know how to convert the posicxt function
+    dplyr::mutate_at(var_datetime, function(x) {
+      x <- dplyr::case_when(
+        x == "" ~ NA,
+        TRUE ~ x
+      )
+      as.POSIXct(x)
+    })
+
+
   #Recalculate calculated fields (previous to transforming factors and other preprocessing)
   #It wil create duplicate variables of each calculated field with "_recalc" in the end and the recalculated value
 
   results <- c(results, stringr::str_glue("{ind}. Recalculating calculated fields and saving them as '[field_name]_recalc'\n\n"))
   ind <- ind + 1
 
-  #If the project is longitudinal and the event hasn't been specified no recalculation was possible
+  #If the project is longitudinal and the event hasn't been specified no recalculation is possible
   if(longitudinal & is.null(event_form)){
 
-    results <- c(results, "\nNo recalculation was possible as the project has more than one event and the event-form correspondence has not been specified\n")
+    results <- c(results, "\nNo recalculation is possible as the project has more than one event and the event-form correspondence has not been specified\n")
 
   }else{
 
-    recalc <- recalculate(data, dic, event_form)
+    recalc <- recalculate(data, dic, event_form, exclude_recalc)
 
     data <- recalc$data
     dic <- recalc$dic
@@ -134,7 +147,7 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
 
 
   if(checkbox_na){
-    results <- c(results, stringr::str_glue("\n\n{ind}. Transforming checkboxes: changing their values to No/Yes and changing their names to the names of its options. For checkboxes that have a branching logic, when the logic isn't satisfied their values will be set to missing\n\n"))
+    results <- c(results, stringr::str_glue("\n\n{ind}. Transforming checkboxes: changing their values to No/Yes and changing their names to the names of its options. For checkboxes that have a branching logic, when the logic isn't satisfied or it's missing their values will be set to missing\n\n"))
   }else{
     results <- c(results, stringr::str_glue("\n\n{ind}. Transforming checkboxes: changing their values to No/Yes and changing their names to the names of its options. For checkboxes that have a branching logic, when the logic is missing their values will be set to missing\n\n"))
   }
@@ -156,7 +169,7 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
 
     #Transform missings of checkboxes with branching logic:
 
-    trans <- transform_checkboxes(data, dic, event_form, checkbox_labels, checkbox_na)
+    trans <- transform_checkboxes(data, dic, event_form, checkbox_na)
 
     data <- trans$data
 
@@ -170,24 +183,24 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
         ~ factor(.x, levels = 0:1, labels = checkbox_labels)
       ))
 
-    #Change the names:
+    #Change the variable names:
 
     data_dic <- checkbox_names(data, dic, labels, checkbox_labels)
 
     data <- data_dic$data
     dic <- data_dic$dic
 
-    #If the project is longitudinal and the event_form hasn't been specified no branching logic evaluation was possible
+    #If the project is longitudinal and the event_form hasn't been specified no branching logic evaluation is possible
     if(longitudinal & is.null(event_form)){
-      results <- c(results, "\nBranching logic evaluation was not possible as the project has more than one event and the event-form correspondence has not been specified\n")
+      results <- c(results, "\nBranching logic evaluation is not possible as the project has more than one event and the event-form correspondence has not been specified\n")
     }
 
   }else{
-    results <- c(results, "\nNo checkboxes have been found in the data\n")
+    results <- c(results, "\nNo checkboxes are found in the data\n")
   }
 
   #Replace original variables with their factor version except for redcap_event_name and redcap_data_access_group
-  #If we dont want to convert another additional variable to factor we can specify it with the exclude argument: to_factor(data, exclude = "var")
+  #If we dont want to convert another additional variable to factor we can specify it with the exclude argument:
 
   results <- c(results, stringr::str_glue("\n\n{ind}. Replacing original variables for their factor version"))
   ind <- ind + 1
@@ -201,32 +214,6 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
     #Fix factors:
     dplyr::mutate_if(is.factor,function(x){levels(x)[levels(x)==""] <- NA; x})
 
-  #Convert dates (dates to as.date and dates & hours to posicxt):
-
-  names_d <- names(data)[grepl("^d_", names(data))]
-  names_posicxt <- names_d[purrr::map_lgl(names_d, function(x){
-    if(any(grepl(":00", data[,x]))){
-      TRUE
-    }else{
-      FALSE
-    }
-  })]
-  names_d <- names_d[!names_d %in% names_posicxt]
-
-  data <- data %>%
-    dplyr::mutate_at(names_d, as.Date) %>%
-    dplyr::mutate_at(names_posicxt, as.POSIXct)
-
-  #Only if we would want a labelled dataset:
-  if(keep_labels){
-    #Restore labels:
-    change_lab<-function(i){
-     labels[i]
-    }
-    Hmisc::label(data)<-purrr::map(1:ncol(data),change_lab)
-  }
-
-
   results <- c(results, stringr::str_glue("\n\n{ind}. Deleting variables that contain some patterns"))
   ind <- ind + 1
 
@@ -243,6 +230,8 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
       }else{
         data <- data %>%
           dplyr::select(!tidyselect::contains(delete_vars[i]))
+        dic <- dic %>%
+          dplyr::filter(!grepl(delete_vars[i], .data$field_name))
       }
 
     }
@@ -250,9 +239,9 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
   }
 
   #Arrange our dataset by record_id and event (will keep the same order of events as in redcap)
-  if(longitudinal){
+  if(longitudinal) {
     data <- data %>%
-      dplyr::arrange(.data$record_id, .data$redcap_event_name.factor)
+      dplyr::arrange(factor(.data$record_id, levels = unique(.data$record_id)), .data$redcap_event_name.factor)
   }
 
 
@@ -320,18 +309,39 @@ rd_transform <- function(..., data = NULL, dic = NULL, event_form = NULL, event_
 
     }
 
-    list(
-      data = data,
-      dictionary = dic,
-      results = stringr::str_glue("{results}")
-    )
+    if(!is.null(event_form)) {
+      list(
+        data = data,
+        dictionary = dic,
+        event_form = event_form,
+        results = stringr::str_glue("{results}")
+      )
+    } else {
+      list(
+        data = data,
+        dictionary = dic,
+        results = stringr::str_glue("{results}")
+      )
+    }
+
+
+  }else {
+
+    if(!is.null(event_form)) {
+      list(
+        data = data,
+        dictionary = dic,
+        event_form = event_form,
+        results = stringr::str_glue("{results}")
+      )
+    } else {
+      list(
+        data = data,
+        dictionary = dic,
+        results = stringr::str_glue("{results}")
+      )
+    }
 
-  }else{
-    list(
-      data = data,
-      dictionary = dic,
-      results = stringr::str_glue("{results}")
-    )
   }
 
 
